@@ -76,6 +76,7 @@ let height b = b.height
 let ctr b = b.ctr
 
 let bpath b = b.contour
+let set_bpath p b = {b with contour = p}
 
 let halfheight b = Point.pt (zero, b.height /./ 2.)
 let halfwidth b = Point.pt (b.width /./ 2., zero)
@@ -208,7 +209,7 @@ let rec draw ?(debug=false) b =
           | Name.Internal _ -> Command.nop
           | Name.Userdef s ->
               Command.label ~pos:`Center
-              (Picture.tex ("\\tiny " ^ (Picture.escape_all s))) 
+              (Picture.tex ("\\tiny " ^ (Picture.escape_all s)))
                 (north_west b)]
     else
       Command.nop
@@ -263,22 +264,23 @@ let no_drawing _ = Command.nop
 let fresh_name =
   let x = ref 1 in
   (fun () ->  incr x; Name.Internal (!x) )
-let mkbox ?(style=Rect) ?dx ?dy ?name ?(stroke=Some Color.black)
+let mkbox ?(style=Rect) ?dx ?dy ?name ?brush ?(stroke=Some Color.black)
           ?pen ?dash ?fill ?(pre_draw=no_drawing) ?(post_draw=no_drawing)
           w h c desc =
   let w,h,s = make_contour style ?dx ?dy w h c in
+  let b = Brush.t ?pen ?dash ?color:stroke ?brush () in
   let name =
     match name with
     | None -> fresh_name ()
     | Some s -> Name.Userdef s in
-  { desc = desc; name = name; stroke = stroke; pen = pen;
-    fill = fill; dash = dash; width = w; height = h;
+  { desc = desc; name = name; stroke = Brush.color b; pen = Brush.pen b;
+    fill = fill; dash = Brush.dash b; width = w; height = h;
     ctr = c; contour = s; post_draw = post_draw;
     pre_draw = pre_draw }
 
-let pic ?style ?dx ?dy ?name ?(stroke=None) ?pen ?dash ?fill pic =
+let pic ?style ?dx ?dy ?name ?brush ?(stroke=None) ?pen ?dash ?fill pic =
   let c = Picture.ctr pic in
-  mkbox ?style ?dx ?dy ?name ~stroke ?pen ?dash ?fill
+  mkbox ?style ?dx ?dy ?name ?brush ~stroke ?pen ?dash ?fill
         (Picture.width pic) (Picture.height pic) c (Pic pic)
 
 let merge_maps =
@@ -289,35 +291,35 @@ let merge_maps =
     NMap.add b.name b m in
   List.fold_left add_one NMap.empty
 
-let box ?style ?dx ?dy ?name ?stroke ?pen ?dash ?fill b =
-  mkbox ?style ?dx ?dy ?name ?stroke ?pen ?dash ?fill
+let box ?style ?dx ?dy ?name ?brush ?stroke ?pen ?dash ?fill b =
+  mkbox ?style ?dx ?dy ?name ?brush ?stroke ?pen ?dash ?fill
   (width b) (height b) (ctr b) (Grp ([|b|], merge_maps [b] ))
 
-let path ?style ?dx ?dy ?name ?(stroke=None) ?pen ?dash ?fill p =
-  pic ?style ?dx ?dy ?name ~stroke ?pen ?dash ?fill
+let path ?style ?dx ?dy ?name ?brush ?(stroke=None) ?pen ?dash ?fill p =
+  pic ?style ?dx ?dy ?name ?brush ~stroke ?pen ?dash ?fill
     (Picture.make (Command.draw p))
 
-let empty ?(width=Num.zero) ?(height=Num.zero) ?style ?name ?(stroke=None)
-          ?pen ?dash ?fill () =
-  mkbox ?style ?name ~dx:zero ~dy:zero ~stroke ?pen ?dash ?fill
+let empty ?(width=Num.zero) ?(height=Num.zero) ?style ?name ?brush
+          ?(stroke=None) ?pen ?dash ?fill () =
+  mkbox ?style ?name ~dx:zero ~dy:zero ?brush ~stroke ?pen ?dash ?fill
     width height Point.origin Emp
 
-let empty_from_box ?style ?name ?(stroke=None) ?pen ?dash ?fill box =
-  mkbox ?style ?name ~stroke ?pen ?dash ?fill
+let empty_from_box ?style ?name ?brush ?(stroke=None) ?pen ?dash ?fill box =
+  mkbox ?style ?name ?brush ~stroke ?pen ?dash ?fill
     (width box) (height box) (ctr box) Emp
 
 
 (* groups the given boxes in a new box *)
 let group ?style ?(dx=Num.zero) ?(dy=Num.zero)
-  ?name ?(stroke=None) ?pen ?dash ?fill bl =
+  ?name ?brush ?(stroke=None) ?pen ?dash ?fill bl =
     let xmin b = xpart (south_west b) in
     let xmax b = xpart (north_east b) in
     let ymin b = ypart (south_west b) in
     let ymax b = ypart (north_east b) in
     match bl with
     | [] -> empty ~width:dx ~height:dy ?style
-                  ?name ~stroke ?pen ?dash ?fill ()
-    | [b] -> box ?style ~dx ~dy ?name ~stroke ?pen ?dash ?fill b
+                  ?name ?brush ~stroke ?pen ?dash ?fill ()
+    | [b] -> box ?style ~dx ~dy ?name ?brush ~stroke ?pen ?dash ?fill b
     | b::r ->
         let xmin,xmax,ymin,ymax =
           List.fold_left
@@ -331,11 +333,11 @@ let group ?style ?(dx=Num.zero) ?(dy=Num.zero)
         let w = xmax -/ xmin in
         let h = ymax -/ ymin in
         let c = Point.pt (xmin +/ w /./ 2., ymin +/ h /./ 2.) in
-        mkbox ?style ~dx ~dy ?name ~stroke ?pen ?dash ?fill w h c
+        mkbox ?style ~dx ~dy ?name ?brush ~stroke ?pen ?dash ?fill w h c
           (Grp (Array.of_list bl, merge_maps bl))
 
-let group_array ?name ?stroke ?fill ?dx ?dy ba =
-  group ?name ?stroke ?fill ?dx ?dy (Array.to_list ba)
+let group_array ?name ?brush ?stroke ?fill ?dx ?dy ba =
+  group ?name ?brush ?stroke ?fill ?dx ?dy (Array.to_list ba)
 
 (* groups the given boxes in a rectangular shape of size [w,h]
    and center [c] *)
@@ -346,7 +348,7 @@ let group_rect ?name ?(stroke=None) w h c bl =
 
 type 'a box_creator =
   ?dx:Num.t -> ?dy:Num.t -> ?name:string ->
-  ?stroke:Color.t option -> ?pen:Pen.t -> ?dash:Dash.t ->
+  ?brush:Brush.t -> ?stroke:Color.t option -> ?pen:Pen.t -> ?dash:Dash.t ->
   ?fill:Color.t -> 'a -> t
 
 let rect = box ~style:Rect
@@ -357,8 +359,8 @@ let patatoid = box ~style:Patatoid
 let patatoid2 = box ~style:Patatoid2
 let round_box = box ~style:RoundBox
 
-let tex ?style ?dx ?dy ?name ?(stroke=None) ?pen ?dash ?fill s =
-  pic ?style ?dx ?dy ?name ~stroke ?pen ?dash ?fill (Picture.tex s)
+let tex ?style ?dx ?dy ?name ?brush ?(stroke=None) ?pen ?dash ?fill s =
+  pic ?style ?dx ?dy ?name ?brush ~stroke ?pen ?dash ?fill (Picture.tex s)
 
 let nth i b = match b.desc with
   | Grp (a, _ ) ->
@@ -390,6 +392,11 @@ let get' n b =
 let get n b = get' (Name.Userdef n) b
 let sub b1 b2 = get' b1.name b2
 
+let relative b g =
+  let b' = sub b g in
+  let v = Point.sub (ctr b) (ctr b') in
+  shift v g
+
 let get_fill b = b.fill
 let set_fill c b = { b with fill = Some c }
 
@@ -410,6 +417,10 @@ let set_name name b = {b with name = Name.Userdef name}
 let set_post_draw f b = {b with post_draw = f}
 let set_pre_draw f b = {b with pre_draw = f}
 
+let add_post_draw f b =
+  let d = b.post_draw in
+  { b with post_draw = (fun t -> Command.append (d t) (f t)) }
+
 let clear_post_draw b = {b with post_draw = no_drawing }
 let clear_pre_draw b = {b with pre_draw = no_drawing }
 
@@ -480,24 +491,20 @@ let set_width pos w b =
       n +/ ((xpart b.ctr -/ n) */ (w // b.width)) in
   { b with width = w; ctr = Point.pt (nc, ypart b.ctr) }
 
-let set_gen2 mycorner chgp pos1 y1 pos2 y2 b =
-  let pos1 = mycorner pos1 b in
-  let pos2 = mycorner pos2 b in
-  let conv x =
-    let a = (y1 -/ y2) // (pos1 -/ pos2) in
-    let b = ((y2 */ pos1) -/ (y1 */ pos2)) // (pos1 -/ pos2) in
-    a */ x +/ b in
-  let h = conv b.height in
-  let ctr = chgp conv b.ctr in
-  { b with height = h; ctr = ctr }
+let set_gen2 mycorner chdim pos1 y1 pos2 y2 box =
+  let pos1 = mycorner pos1 box in
+  let pos2 = mycorner pos2 box in
+  let a = (y1 -/ y2) // (pos1 -/ pos2) in
+  let b = ((y2 */ pos1) -/ (y1 */ pos2)) // (pos1 -/ pos2) in
+  let w,h = chdim (fun x -> a */ x) (box.width,box.height) in
+  let ctr = chdim (fun x -> a */ x +/ b) (xpart box.ctr,ypart box.ctr) in
+  { box with width = w; height = h; ctr = Point.pt ctr }
 
 let set_height2 pos1 y1 pos2 y2 b = set_gen2 cornerv
-  (fun conv p -> Point.pt (xpart p, conv (ypart p)))
-  pos1 y1 pos2 y2 b
+  (fun conv (x,y) -> (x, conv y)) pos1 y1 pos2 y2 b
 
 let set_width2 pos1 y1 pos2 y2 b = set_gen2 cornerh
-  (fun conv p -> Point.pt (conv (xpart p),ypart p))
-  pos1 y1 pos2 y2 b
+  (fun conv (x,y) -> (conv x,y)) pos1 y1 pos2 y2 b
 
 let valign ?(pos=`Center) x l =
   List.map (fun b -> shift (Point.pt (x -/ xcoord pos b, zero)) b) l
@@ -526,7 +533,7 @@ let max_width l = Num.fold_max width Num.zero l
 let same_size ?(pos=`Center) bl =
   List.map (set_size pos ~width:(max_width bl) ~height:(max_height bl)) bl
 
-let same_height ?(pos=`Center) bl = 
+let same_height ?(pos=`Center) bl =
   List.map (set_height pos (max_height bl)) bl
 let same_width ?(pos=`Center) bl = List.map (set_width pos (max_width bl)) bl
 
@@ -582,12 +589,12 @@ let hequalize h l = List.map (set_height h) l
 let wequalize w l = List.map (set_width w) l
 
 let hbox ?padding ?pos ?style ?min_width ?same_width ?dx ?dy
-         ?name ?stroke ?pen ?dash ?fill l =
-  group ?style ?dx ?dy ?name ?stroke ?pen ?dash ?fill
+         ?name ?brush ?stroke ?pen ?dash ?fill l =
+  group ?style ?dx ?dy ?name ?brush ?stroke ?pen ?dash ?fill
     (hbox_list ?padding ?pos ?min_width ?same_width l)
 
 let vbox ?padding ?pos ?style ?min_height ?same_height ?dx ?dy
-         ?name ?stroke ?pen ?dash ?fill l =
+         ?name ?brush ?stroke ?pen ?dash ?fill l =
   group ?style ?dx ?dy ?name ?stroke ?pen ?dash ?fill
     (vbox_list ?padding ?pos ?min_height ?same_height l)
 
@@ -609,7 +616,7 @@ let vblock ?padding ?(pos=`Center) ?name ?stroke ?pen ?dash
            ?min_height ?same_height pl =
   group ?name
     (List.map (modify_box ?stroke ?pen ?dash)
-      (vbox_list ~pos ?min_height ?same_height
+      (vbox_list ?padding ~pos ?min_height ?same_height
         (List.map (set_width (extracth pos) (max_width pl)) pl)))
 
 
@@ -672,6 +679,20 @@ let gridi ?hpadding ?vpadding ?pos ?stroke ?pen ?dash w h f =
   let m = Array.init h (fun j -> Array.init w (fun i -> f i j)) in
   grid ?hpadding ?vpadding ?pos ?stroke ?pen ?dash m
 
+let henlarge l =
+  let toh f x = xpart (f x) in
+  let min = Num.fold_min (toh west) (bp infinity) l in
+  let max = Num.fold_max (toh east) (bp neg_infinity) l in
+  List.map (set_width2 `West min `East max) l
+
+
+let venlarge l =
+  let tow f x = ypart (f x) in
+  let min = Num.fold_min (tow south) (bp infinity) l in
+  let max = Num.fold_max (tow north) (bp neg_infinity) l in
+  List.map (set_height2 `North max `South min) l
+
+
 module P = Path
 
 let strip ?sep p = match sep with
@@ -683,15 +704,15 @@ let cpath ?style ?outd ?ind ?sep a b =
   let p = P.pathk ?style [P.knotp ?r (ctr a); P.knotp ?l (ctr b)] in
   strip ?sep (P.cut_after (bpath b) (P.cut_before (bpath a) p))
 
-let cpath_left ?style ?outd ?ind a b =
+let cpath_left ?style ?outd ?ind ?sep a b =
   let r,l = outd, ind in
   let p = P.pathk ?style [P.knotp ?r (ctr a); P.knotp ?l b] in
-  P.cut_before (bpath a) p
+  strip ?sep (P.cut_before (bpath a) p)
 
-let cpath_right ?style ?outd ?ind a b =
+let cpath_right ?style ?outd ?ind ?sep a b =
   let r,l = outd, ind in
   let p = P.pathk ?style [P.knotp ?r a; P.knotp ?l (ctr b)] in
-  P.cut_after (bpath b) p
+  strip ?sep (P.cut_after (bpath b) p)
 
 (* (* Deleted because of circular dependency with the Arrow module.
 It did not seem to be used anyway. *)
@@ -706,7 +727,7 @@ let thick_arrow ?style ?(boxed=true) ?line_color ?fill_color ?outd ?ind ?width
 
 (* Specials Points *)
 let setp name pt box =
-  let add_smap m = 
+  let add_smap m =
     NMap.add (Name.Userdef name) (shift pt (empty ~name ())) m in
   { box with
     desc =
@@ -757,6 +778,6 @@ let place posa ?(pos = opposite_position posa) ?padding a b =
     | Some padding ->
         shift (Point.mult padding (normalize (Point.sub pa (ctr a)))) c
 
-(* Boxlike *) 
+(* Boxlike *)
 
 let set_pos = center
