@@ -66,14 +66,35 @@ type page = {
   commands : command list
 }
 
+type fontmap = Dvi_util.font_def Int32Map.t
 type t = {
   preamble : preamble;
   pages : page list;
   postamble : postamble;
   postpostamble : postpostamble;
-  font_map : Fonts.font_def Int32Map.t
+  font_map : fontmap
 }
 
+(* vf *)
+
+type preamble_vf = {
+  pre_vf_version : int;
+  pre_vf_text    : string;
+  pre_vf_cs      : int32;
+  pre_vf_ds      : float;
+}
+
+type char_desc =
+    { char_code       : int32;
+      char_tfm        : int32;
+      char_commands   : command list}
+
+type vf =
+    { vf_preamble   : preamble_vf;
+      vf_font_map   : fontmap;
+      vf_chars_desc : char_desc list}
+(* *)
+
 let fontmap d = d.font_map
 
 module Print = struct
@@ -81,41 +102,101 @@ module Print = struct
   let preamble fmt p =
     fprintf fmt "* Preamble :\n";
     fprintf fmt "\tversion number = %d\n" p.pre_version;
-    fprintf fmt "\tnumerator/denominator = %ld/%ld\n" 
+    fprintf fmt "\tnumerator/denominator = %ld/%ld\n"
       p.pre_num p.pre_den;
     fprintf fmt "\tmagnification = %ld\n" p.pre_mag;
     fprintf fmt "\tcomment : %s\n" p.pre_text
 
-  let page fmt 
-      {counters = c; previous = prev; commands = cmds} =
-    fprintf fmt "* Page number :";
-    Array.iter (fun c -> fprintf fmt "%ld;" c) c; fprintf fmt "\n";
-    fprintf fmt "\tPrevious page can be found at %ld\n" prev;
-    fprintf fmt "\t<list of commands skipped ...>"
-      
-  let pages fmt =
-    List.iter (fun p -> fprintf fmt "%a\n" page p)
-
   let fonts fmt fonts =
     fprintf fmt "* Fonts defined in this file :\n";
-    Int32Map.iter (fun k f -> Fonts.Print.font k fmt f) fonts
+    Int32Map.iter (fun k f -> Print_font.font k fmt f) fonts
 
   let postamble fmt p =
     fprintf fmt "* Postamble :\n";
     fprintf fmt "\tlast page = %ld\n" p.last_page;
-    fprintf fmt "\tnumerator/denominator = %ld/%ld\n" 
+    fprintf fmt "\tnumerator/denominator = %ld/%ld\n"
       p.post_num p.post_den;
     fprintf fmt "\tmagnification = %ld\n" p.post_mag;
-    fprintf fmt "\theight - width = %ld - %ld\n" 
+    fprintf fmt "\theight - width = %ld - %ld\n"
       p.post_height p.post_width;
     fprintf fmt "\tmaximum stack depth = %d\n"  p.post_stack;
-    fprintf fmt "\ttotal # of pages = %d\n"  p.post_pages  
+    fprintf fmt "\ttotal # of pages = %d\n"  p.post_pages
 
   let postpostamble fmt p =
     fprintf fmt "* Postpostamble :\n";
     fprintf fmt "\tPostamble can be found at %ld.\n" p.postamble_pointer;
     fprintf fmt "\tDVI version : %d\n" p.post_post_version
-      
+
+
+  let commands fmt = function
+    | SetChar c ->
+        Format.fprintf fmt "Setting character %ld.@\n" c
+    | SetRule(a, b) ->
+        Format.fprintf fmt "Setting rule (w=%ld, h=%ld).@\n" a b
+    | PutChar c ->
+        Format.fprintf fmt "Putting character %ld.@\n" c
+    | PutRule(a, b) ->
+        Format.fprintf fmt "Putting rule (w=%ld, h=%ld).@\n" a b
+    | Push ->
+        Format.fprintf fmt "Push current state.@\n"
+    | Pop ->
+        Format.fprintf fmt "Pop current state.@\n"
+    | Right b ->
+        Format.fprintf fmt "Moving right %ld.@\n" b
+    | Wdefault ->
+        Format.fprintf fmt "Moving right by the default W.@\n"
+    | W b ->
+        Format.fprintf fmt "Moving right and changing W to %ld.@\n" b
+    | Xdefault ->
+        Format.fprintf fmt "Moving right by the default X.@\n"
+    | X b ->
+        Format.fprintf fmt "Moving right and changing X to %ld.@\n" b
+    | Down a ->
+        Format.fprintf fmt "Moving down %ld.@\n" a
+    | Ydefault ->
+        Format.fprintf fmt "Moving down by the default Y.@\n"
+    | Y a ->
+        Format.fprintf fmt "Moving down and changing Y to %ld.@\n" a
+    | Zdefault ->
+        Format.fprintf fmt "Moving down by the default Z.@\n"
+    | Z a ->
+        Format.fprintf fmt "Moving down and changing Z to %ld.@\n" a
+    | FontNum f ->
+        Format.fprintf fmt "Font is now set to %ld@\n" f
+    | Special xxx ->
+        Format.fprintf fmt "Special command : %s@\n" xxx
+
+  let print_chars fmt c =
+    Format.fprintf fmt "@[<hov 3>%ld : %a@]@\n" c.char_code
+      (Misc.print_list Misc.newline commands) c.char_commands
+
+  let print_chars_desc = (Misc.print_list Misc.newline print_chars)
+
+  let print_vf fmt vf =
+    Format.fprintf fmt "cs=%ld ds=%f %s@\n%a@\n%a@\n"
+      vf.vf_preamble.pre_vf_cs
+      vf.vf_preamble.pre_vf_ds
+      vf.vf_preamble.pre_vf_text
+      fonts vf.vf_font_map
+      print_chars_desc vf.vf_chars_desc
+
+  let page verbose fmt
+      {counters = c; previous = prev; commands = cmds} =
+    fprintf fmt "* Page number :";
+    Array.iter (fun c -> fprintf fmt "%ld;" c) c; fprintf fmt "\n";
+    fprintf fmt "\tPrevious page can be found at %ld\n" prev;
+    if verbose then Misc.print_list Misc.newline commands fmt cmds
+    else fprintf fmt "\t<list of commands skipped ...>"
+
+  let pages verbose fmt =
+    List.iter (fun p -> fprintf fmt "%a\n" (page verbose) p)
+
+  let page_verb = page true
+  let pages_verb = pages true
+
+  let page = page false
+  let pages = pages false
+
   let doc name fmt doc =
     fprintf fmt "***********************\n";
     fprintf fmt "Reading DVI file : %s\n" name;
@@ -129,7 +210,7 @@ end
 exception DviError of string ;;
 let dvi_error s = raise (DviError s)
 
-let preamble bits = 
+let preamble bits =
   bitmatch bits with
     | { 247 : 8;              (* Preamble opcode *)
 	version : 8;          (* DVI version *)
@@ -147,7 +228,7 @@ let preamble bits =
     | { _ : -1 : bitstring } ->
 	dvi_error "Ill-formed preamble"
 
-let add_font k font map = 
+let add_font k font map =
   if Int32Map.mem k map then
     dvi_error "Redefinition of font not allowed"
   else
@@ -163,7 +244,7 @@ let font_def bits =
 	name : (a+l)*8 : string;   (* the full name w/ area *)
 	bits : -1 : bitstring
       } ->
-        Fonts.mk_font_def ~checksum ~scale_factor ~design_size
+        mk_font_def ~checksum ~scale_factor ~design_size
           ~area:(String.sub name 0 a) ~name:(String.sub name a l), bits
     | { _ : -1 : bitstring } ->
 	dvi_error "Ill_formed font definition"
@@ -193,11 +274,14 @@ let signed i j unsigned =
   else
     Int32.logor unsigned j
 
-let signed_8 = signed (Int32.shift_left Int32.one 23) (Int32.logxor Int32.minus_one (Int32.of_int 0xff))
-let signed_16 = signed (Int32.shift_left Int32.one 15) (Int32.logxor Int32.minus_one (Int32.of_int 0xffff))
-let signed_24 = signed (Int32.shift_left Int32.one 23) (Int32.logxor Int32.minus_one (Int32.of_int 0xffffff))
-	
-let command bits = 
+let signed_8 = signed (Int32.shift_left Int32.one 23)
+  (Int32.logxor Int32.minus_one (Int32.of_int 0xff))
+let signed_16 = signed (Int32.shift_left Int32.one 15)
+  (Int32.logxor Int32.minus_one (Int32.of_int 0xffff))
+let signed_24 = signed (Int32.shift_left Int32.one 23)
+  (Int32.logxor Int32.minus_one (Int32.of_int 0xffffff))
+
+let command bits =
   bitmatch bits with
       (* Setting Characters *)
     | { k : 8 ; bits : -1 : bitstring } when 0 <= k && k <= 127 ->
@@ -311,7 +395,7 @@ let command bits =
 	Special x, bits
     | { 241 : 8; k : 24; x : k * 8 : string; bits : -1 : bitstring } ->
 	Special x, bits
-    | { 242 : 8; k : 32; x : (Int32.to_int k) * 8 : string; 
+    | { 242 : 8; k : 32; x : (Int32.to_int k) * 8 : string;
 	bits : -1 : bitstring } ->
 	Special x, bits
     | { _ : -1 : bitstring } ->
@@ -321,10 +405,10 @@ let rec page commands fonts bits =
   bitmatch bits with
     | { 140 : 8; bits : -1 : bitstring } -> (* End of page opcode *)
 	commands, fonts, bits
-      (* nop opcode *) 
+      (* nop opcode *)
     | { 138 : 8; bits : -1 : bitstring } ->
 	page commands fonts bits
-      (* font definitions *) 
+      (* font definitions *)
     | { 243 : 8; k : 8; bits : -1 : bitstring } ->
 	let font, bits = font_def bits in
 	  page commands (add_font (Int32.of_int k) font fonts) bits
@@ -342,12 +426,14 @@ let rec page commands fonts bits =
 	let cmd, bits = command bits in
 	  page (cmd::commands) fonts bits
 
+let page = page []
+
 let rec pages p fonts bits =
   bitmatch bits with
-      (* nop opcode *) 
+      (* nop opcode *)
     | { 138 : 8; bits : -1 : bitstring } ->
 	pages p fonts bits
-      (* font definitions *) 
+      (* font definitions *)
     | { 243 : 8; k : 8; bits : -1 : bitstring } ->
 	let font, bits = font_def bits in
 	  pages p (add_font (Int32.of_int k) font fonts) bits
@@ -363,22 +449,25 @@ let rec pages p fonts bits =
       (* begin page opcode *)
     | { 139 : 8; bits : -1 : bitstring } ->
 	let counters, previous, bits = page_counters bits in
-	let cmds, fonts, bits = page [] fonts bits in
-	let newp = 
-	  {counters = counters; previous = previous; commands = cmds} in
+	let cmds, fonts, bits = page fonts bits in
+	let newp =
+	  {counters = counters; previous = previous; commands = List.rev cmds}
+        in
         (* Pages in reverse order *)
 	  pages (newp::p) fonts bits
     | { bits : -1 : bitstring } ->
 	p, fonts, bits
 	(* dvi_error "Expected : nop, font_definition, or new page" *)
 
-let postamble bits = 
+let read_pages = pages []
+
+let postamble bits =
   let rec skip_font_defs bits =
     bitmatch bits with
-	(* nop opcode *) 
+	(* nop opcode *)
       | { 138 : 8; bits : -1 : bitstring } ->
 	  skip_font_defs bits
-        (* font definitions *) 
+        (* font definitions *)
       | { 243 : 8; k : 8; bits : -1 : bitstring } ->
 	  let _, bits = font_def bits in skip_font_defs bits
       | { 244 : 8; k : 16 : bigendian; bits : -1 : bitstring } ->
@@ -405,7 +494,7 @@ let postamble bits =
 	{ last_page = last_page;
 	  post_num = num; post_den = den; post_mag = mag;
 	  post_height = height; post_width = width;
-	  post_stack = stack; post_pages = pages 
+	  post_stack = stack; post_pages = pages
 	}, skip_font_defs bits
     | { _ : -1 : bitstring } ->
 	dvi_error "Ill-formed postamble"
@@ -437,7 +526,7 @@ let postpostamble bits =
 let read_file file =
   let bits = Bitstring.bitstring_of_file file in
   let preamble, bits = preamble bits in
-  let pages, fonts, bits = pages [] Int32Map.empty bits in
+  let pages, fonts, bits = read_pages Int32Map.empty bits in
   let postamble, bits = postamble bits in
   let postpostamble = postpostamble bits in
     { preamble = preamble;
@@ -447,11 +536,94 @@ let read_file file =
       font_map = fonts
     }
 
-let get_conv doc = 
+
+let vf_preamble bits =
+  bitmatch bits with
+    | { 247 : 8;              (* Preamble opcode *)
+	version : 8;          (* VF version *)
+	k : 8;                (* size of string x *)
+	x : 8*k : string;     (* file comment *)
+	cs : 32 : bigendian; (* denominator *)
+	ds : 32 : bigendian; (* magnification *)
+	bits : -1 : bitstring
+      } ->
+	{ pre_vf_version = version;
+	  pre_vf_text = x;
+          pre_vf_cs   = cs;
+          pre_vf_ds   = (Int32.to_float ds) /. (2.**20.)
+	}, bits
+    | { _ : -1 : bitstring } ->
+	dvi_error "Ill-formed preamble"
+
+(* Could factor the code with the one of page?*)
+let rec preamble_fonts fonts bits =
+  bitmatch bits with
+      (* font definitions *)
+    | { 243 : 8; k : 8; bits : -1 : bitstring } ->
+	let font, bits = font_def bits in
+	  preamble_fonts (add_font (Int32.of_int k) font fonts) bits
+    | { 244 : 8; k : 16 : bigendian; bits : -1 : bitstring } ->
+	let font, bits = font_def bits in
+	  preamble_fonts (add_font (Int32.of_int k) font fonts) bits
+    | { 245 : 8; k : 24 : bigendian; bits : -1 : bitstring } ->
+	let font, bits = font_def bits in
+	  preamble_fonts (add_font (Int32.of_int k) font fonts) bits
+    | { 246 : 8; k : 32 : bigendian; bits : -1 : bitstring } ->
+	let font, bits = font_def bits in
+	  preamble_fonts (add_font k font fonts) bits
+    | { bits : -1 : bitstring } ->
+      fonts,bits
+
+let rec command_list cmds bits =
+  if Bitstring.bitstring_length bits = 0 then cmds
+  else let cmd,bits = command bits in
+       command_list (cmd::cmds) bits
+
+let rec vf_chars chars bits =
+   bitmatch bits with
+    | {248 : 8} -> chars
+    | {242 : 8;
+       pl  : 32;
+       cc  : 32;
+       tfm : 32;
+       _ : (Int32.to_int pl)*8 : bitstring;
+       bits : -1 : bitstring
+      } ->
+      let char = { char_code = cc;
+                   char_tfm  = tfm;
+                   char_commands = []} in
+      vf_chars (char::chars) bits
+    | {pl : 8;
+       cc  : 8;
+       tfm : 24;
+       coms : pl*8 : bitstring;
+       bits : -1 : bitstring
+      } ->
+      let coms = List.rev (command_list [] coms) in
+      let char = { char_code = Int32.of_int cc;
+                   char_tfm  = Int32.of_int tfm;
+                   char_commands = coms} in
+      vf_chars (char::chars) bits
+    | {i : 8} -> dvi_error "vf : ill-formed character"
+
+let print_vf = Print.print_vf
+
+let read_vf_file file =
+  let bits = Bitstring.bitstring_of_file file in
+  let preamble, bits = vf_preamble bits in
+  let fonts, bits = preamble_fonts Int32Map.empty bits in
+  let chars_desc = List.rev (vf_chars [] bits) in
+  {vf_preamble = preamble;
+   vf_font_map    = fonts;
+   vf_chars_desc    = chars_desc}
+
+let get_conv_from_preamble p =
     let formule_magique_cm mag num den =
-      ((Int32.to_float mag) *. ((Int32.to_float num) /. (Int32.to_float den))) /. (10.**8.) in
-    formule_magique_cm doc.preamble.pre_mag 
-      doc.preamble.pre_num doc.preamble.pre_den
+      ((Int32.to_float mag) *.
+          ((Int32.to_float num) /. (Int32.to_float den))) /. (10.**8.) in
+    formule_magique_cm p.pre_mag p.pre_num p.pre_den
+
+let get_conv doc = get_conv_from_preamble doc.preamble
 
 let get_height_cm doc =
   (get_conv doc) *. (Int32.to_float doc.postamble.post_height)
@@ -461,3 +633,28 @@ let get_width_cm doc =
 
 let pages d = d.pages
 let commands p = p.commands
+
+module Incremental = struct
+  type t =
+    { mutable fonts : fontmap;
+      mutable preamble : preamble;
+      chan : in_channel;
+      mutable bits : string * int * int;
+  }
+
+  let mk_t c =
+    let bits = Bitstring.bitstring_of_chan c in
+    let preamble, bits = preamble bits in
+    let pgs, fonts, bits = read_pages Int32Map.empty bits in
+    { fonts = fonts ; preamble = preamble ; bits = bits; chan = c }, pgs
+
+  let next_pages t =
+    let nextbits = Bitstring.bitstring_of_chan t.chan in
+    let pgs, fonts, bits =
+      read_pages t.fonts (Bitstring.concat [t.bits; nextbits]) in
+    t.fonts <- fonts; t.bits <- bits;
+    pgs
+
+  let get_conv i = get_conv_from_preamble i.preamble
+  let font_map i = i.fonts
+end
