@@ -51,7 +51,8 @@ let tree6 =
 		node "3" [leaf "6"; leaf "7"]])
 (*parse >> <<tree7 *)
 let tree7 =
-  let node s = node ~arrow_style:Undirected ~edge_style:HalfSquare (Box.tex s) in
+  let node s = 
+    node ~arrow_style:Undirected ~edge_style:HalfSquare (Box.tex s) in
   let leaf s = leaf (Box.tex s) in
   draw 
      (node "1" [node "2" [node "3" [leaf "4"]; leaf "5"]; 
@@ -89,7 +90,8 @@ let rec bin = function
       Node (n, t :: l)
 
 let rec trans (Node (n,l)) = 
-  node ~arrow_style:Undirected (tex (sprintf "${2^{%d}}$" n)) (List.map trans l)
+  node ~arrow_style:Undirected (tex (sprintf "${2^{%d}}$" n)) 
+    (List.map trans l)
 
 let tree10 = draw (trans (bin 4))
 (*parse >> <<tree11 *)
@@ -112,10 +114,51 @@ let rec to_tree (Node (b,l)) =
 
 
 let tree11 = draw (to_tree (bin 5))
+
+(*parse >> <<tree12 *)
+open Tree_adv
+open Tree_adv.Overlays
+
+let (++) = Command.(++)
+let rec to_tree = function
+  | Node (e,l) -> Tree.node e (List.map to_tree l)
+
+let node i e l = Node ([Aft i,tex e],l)
+
+let place_not_simple t =
+  let module P = Place(Overlays_Boxlike(Box)) in
+  let bplace t = Tree.to_box (to_tree t) in
+  P.gen_place ~place:bplace t
+
+let tree_adv_draw t i =
+  let keep e = try Some (assoq i e)
+  with Not_found ->  None in
+  let t = filter_option keep t in
+  let t_arrow = gen_draw_arrows Command.nop
+    ~style:Helpers.draw_simple_arrow
+    ~corner:Box.corner t in
+  (Tree_adv.draw (fun x -> x) t) ++
+    (fold (++) Command.nop t_arrow)
+
+let mytree_placed = 
+  let t = node 0 "A" 
+    [ node 0 "B" [node 0 "D" [];node 0 "E" []];
+      node 0 "C" [node 1 "F" [];node 2 "G" []]] in
+  place_not_simple t
+  
+
+
+let tree12 = tree_adv_draw mytree_placed 0
+(*parse >> <<tree13 *)
+let tree13 = tree_adv_draw mytree_placed 1
+(*parse >> <<tree14 *)
+let tree14 = tree_adv_draw mytree_placed 2
+
 (*parse >> *)
 
 let _ = 
-  List.iter (fun (name,fig) -> Metapost.emit name fig)
+  List.iter (fun (name,fig) -> Metapost.emit name 
+               (Picture.scale (Num.bp 3.) fig))
   [ "tree1", tree1;
     "tree2", tree2;
     "tree3", tree3;
@@ -127,4 +170,7 @@ let _ =
     "tree9", tree9;
     "tree10", tree10;
     "tree11", tree11;
+    "tree12", tree12;
+    "tree13", tree13;
+    "tree14", tree14;
   ]
