@@ -13,22 +13,43 @@
 (*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                  *)
 (*                                                                        *)
 (**************************************************************************)
-(** This is Mlpost *)
 
 module Signature : sig
   type point
-  type num
 
   module type Boxlike =
   sig
     type t
-    val width : t -> num
-    val height : t -> num
+    val width : t -> float
+    val height : t -> float
     val set_pos : point -> t -> t
   end
 
 end
 
+module File : sig
+  type t
+  val from_string : string -> t
+  val to_string : t -> string
+end
+
+module Ctypes : sig
+
+  IFDEF CAIRO THEN
+    type matrix = Cairo.matrix =
+      { xx : float; yx : float; xy : float; yy : float; x0 : float; y0 : float;}
+
+    type point = Cairo.point =
+        {x : float;
+         y : float}
+  ELSE
+    type matrix =
+      { xx : float; yx : float; xy : float; yy : float; x0 : float; y0 : float;}
+
+    type point = {x : float; y : float}
+  END
+end
+
 
 (** {2 Interfaces to basic Metapost datatypes} *)
 
@@ -42,7 +63,7 @@ module Num : sig
     [float].
   *)
 
-  type t = Signature.num
+  type t = float
       (** The Mlpost numeric type is an abstract datatype *)
 
   (** {2 Conversion functions} *)
@@ -155,6 +176,13 @@ module Color : sig
     (** [cmyk c m y k] constructs the color that corresponds to the color code
 	CMYK(c,m,y,k)  *)
 
+  (** WARNING : If you use transparency with .mps file in your latex
+      document you need to add : \LoadMetaPostSpecialExtensions in the
+      preamble *)
+
+  val rgba : float -> float -> float -> float -> t
+    (** similar to [rgb], but takes the factor of transparency *)
+
   val rgba : float -> float -> float -> float -> t
     (** similar to [rgb], but takes the factor of transparency *)
   val rgb8a : int -> int -> int -> int -> t
@@ -363,7 +391,7 @@ and MetaPath : sig
   type knot = Path.knot
 
   (** Build a knot from a point; the optional arguments are the
-      incoming directions *)
+      incoming directions.Warning they are going in the same direction. *)
   val knotp :
     ?l:direction -> ?r:direction -> Point.t -> knot
 
@@ -494,7 +522,7 @@ and Path : sig
   type knot
 
   (** Build a knot from a point; the optional arguments are the
-      incoming directions *)
+      incoming directions. Warning they are going in the same direction. *)
   val knotp :
     ?l:direction -> ?r:direction -> Point.t -> knot
 
@@ -723,7 +751,7 @@ and Dash : sig
   (** This module permits to define dash patterns, that are used to draw lines
    in different styles *)
 
-  type t
+  type t = Brush.Dash.t
     (** The abstract type of dash patterns *)
 
   val evenly : t
@@ -785,7 +813,7 @@ and Brush : sig
     (** This module permits to define dash patterns, that are used to
         draw lines in different styles *)
 
-    type t (**/**) = Dash.t (**/**)
+    type t
       (** The abstract type of dash patterns *)
 
     val evenly : t
@@ -815,8 +843,10 @@ and Brush : sig
   val t :
     ?color:Color.t -> ?pen:Pen.t ->
     ?dash:Dash.t -> ?scale:Num.t ->
-    ?brush:t -> (* use the value of brush for the default value *)
+    ?brush:t ->
     unit -> t
+  (** [t ~color ~pen ~dash ~scale ~brush] create a brush with color [color] if
+      TODO *)
 
   val pen : t -> Pen.t option
   val dash : t -> Dash.t option
@@ -898,9 +928,7 @@ and Transform : sig
   val rotate_around : Point.t -> float -> t'
     (** Rotate an object by an angle given in degrees, around a given point *)
 
-  type matrix  =
-      { xx : Num.t; yx : Num.t;
-        xy : Num.t; yy : Num.t; x0 : Num.t; y0 : Num.t}
+  type matrix  = Ctypes.matrix
 
   val explicit : matrix -> t'
 
@@ -1019,10 +1047,6 @@ end
 (** Basic drawing commands *)
 and Command : sig
 
-  (** Set the verbosity of all mlpost *)
-  val set_verbosity : bool -> unit
-
-
   (** General Commands to build figures *)
 
   type t
@@ -1196,7 +1220,7 @@ module Box : sig
 
   type 'a box_creator =
     ?dx:Num.t -> ?dy:Num.t -> ?name:string ->
-    ?stroke:Color.t option -> ?pen:Pen.t -> ?dash:Dash.t ->
+    ?brush:Brush.t -> ?stroke:Color.t option -> ?pen:Pen.t -> ?dash:Dash.t ->
     ?fill:Color.t -> 'a -> t
     (** All functions used to create boxes take the following optional
 	parameters : [dx] (resp. [dy]) is the horizontal
@@ -1211,12 +1235,12 @@ module Box : sig
 
   val empty :
     ?width:Num.t -> ?height:Num.t -> ?style:style -> ?name:string ->
-    ?stroke:Color.t option -> ?pen:Pen.t -> ?dash:Dash.t -> ?fill:Color.t ->
-    unit -> t
+    ?brush:Brush.t -> ?stroke:Color.t option -> ?pen:Pen.t ->
+    ?dash:Dash.t -> ?fill:Color.t -> unit -> t
     (** the empty box *)
 
   val empty_from_box :
-    ?style:style -> ?name:string ->
+    ?style:style -> ?name:string -> ?brush:Brush.t ->
     ?stroke:Color.t option -> ?pen:Pen.t -> ?dash:Dash.t -> ?fill:Color.t -> t
     -> t
     (** the empty box with the same position and dimension as the box *)
@@ -1281,6 +1305,8 @@ module Box : sig
 
   (** Get the bounding path of a box *)
   val bpath : t -> Path.t
+  (** Set the bounding path of a box *)
+  val set_bpath : Path.t -> t -> t
 
   (** {2 Special points on a box} *)
 
@@ -1493,6 +1519,17 @@ module Box : sig
         direction will change according to the width / height ratio of
         [a]. *)
 
+
+  val relative : t -> t -> t
+  (** After using one of the previous function which give you a group
+      [g] of box from multiple box (one of them must be [b]), you can
+      use [relative b g] to translate [g] so that [b] inside [g] is
+      at the same place than before.
+      Indeed it's just {!sub} and a translation.
+
+      {!relative} is really useful when someone make a slideshow from a
+      succession of figures. *)
+
   (** {2 Sub-boxes accessors} *)
 
   val nth : int -> t -> t
@@ -1557,7 +1594,16 @@ module Box : sig
   val same_width : ?pos:hposition -> t list -> t list
   val same_size : ?pos:position -> t list -> t list
 
+  val henlarge : t list -> t list
+  (** [henlarge l] set the west boundaries of the box in [l] to the
+      westest boundaries of the box in [l]. same for the east
+      boundaries *)
+
+  val venlarge : t list -> t list
+   (** same as {!henlarge} for vertical boundaries *)
+
   val set_post_draw : (t -> Command.t) -> t -> t
+  val add_post_draw : (t -> Command.t) -> t -> t
   val clear_post_draw : t -> t
   val set_pre_draw : (t -> Command.t) -> t -> t
   val clear_pre_draw : t -> t
@@ -1577,14 +1623,18 @@ module Box : sig
   val cpath_left :
     ?style:Path.joint ->
     ?outd:Path.direction ->
-    ?ind:Path.direction -> t -> Point.t -> Path.t
+    ?ind:Path.direction ->
+    ?sep:Num.t ->
+    t -> Point.t -> Path.t
     (** the path that connects a box and a point and stops at the box
         boundaries *)
 
   val cpath_right :
     ?style:Path.joint ->
     ?outd:Path.direction ->
-    ?ind:Path.direction -> Point.t -> t -> Path.t
+    ?ind:Path.direction ->
+    ?sep:Num.t ->
+    Point.t -> t -> Path.t
     (** the path that connects a box and a point and stops at the box
         boundaries *)
 
@@ -1602,6 +1652,89 @@ module Box : sig
     (** same as center *)
 end
 
+(** Tree-like, triangular shapes,
+    with a root on top and an horizontal bottom line. *)
+module Triangle : sig
+
+  type t = Box.t
+  (** A triangular shape is a box.
+      In the following, we simply call it a "triangle". *)
+
+  val create:
+    ?brush:Brush.t ->
+    ?stroke:Color.t option ->
+    ?pen:Pen.t ->
+    ?dash:Dash.t ->
+    ?fill:Color.t ->
+    ?left:float -> ?right:float -> ?width:Num.t ->
+    depth:Num.t -> unit -> Box.t
+    (** [create depth] creates a triangular shape of depth [depth].
+        The width is equal to [depth], unless specified using [width].
+        [left] and [right] control the horizontal position of the root;
+        they default to [0.5] i.e. the root is centered. Negative values for
+        [left] and [right] are allowed. *)
+
+  val pic:
+    ?brush:Brush.t ->
+    ?stroke:Color.t option ->
+    ?pen:Pen.t ->
+    ?dash:Dash.t ->
+    ?fill:Color.t ->
+    ?dx:float -> ?dy:float -> Picture.t -> t
+    (** builds a triangle surrounding a picture *)
+  val tex:
+    ?brush:Brush.t ->
+    ?stroke:Color.t option ->
+    ?pen:Pen.t ->
+    ?dash:Dash.t ->
+    ?fill:Color.t -> ?dx:float -> ?dy:float -> string -> t
+    (** builds a triangle surrounding a LaTeX label *)
+
+  val root: t -> Point.t
+  val bottom_left: t -> Point.t
+  val bottom_right: t -> Point.t
+    (** Special points in a triangle. *)
+
+  val draw: ?debug:bool -> t -> Command.t
+
+  val root_label: ?pos:Command.position -> Picture.t -> t -> t
+  val tex_root_label: ?pos:Command.position -> string -> t -> t
+    (** Attach a label to the root. *)
+
+  val x_depth: ?x:float -> ?depth:float -> t -> Point.t
+    (** a point inside a triangle is denoted using two coordinates
+        [x] and [depth]. Coordinate [x] controls horizontal placement,
+        with 0.0 for left and 1.0 for right (negative values are allowed).
+        Coordinate [depth] controls vertical placement, with 0.0 for root
+        and 1.0 for bottom line (negative values are allowed).
+        [x] defaults to [0.5] (centered) and [depth] defaults to [1.0]
+        (bottom). *)
+
+  val label:
+    ?x:float -> ?depth:float -> ?pos:Command.position ->
+    Picture.t -> t -> t
+  val tex_label:
+    ?x:float -> ?depth:float -> ?pos:Command.position ->
+    string -> t -> t
+    (** Attach a label to the point (x, depth).
+        Coordinate [depth] defaults to 0.7, so that label is likely to fit
+        nicely in the triangle (otherwise, you may consider using functions
+        [pic] and [tex] above). *)
+
+  val anchor: ?x:float -> ?depth:float -> t -> t -> t
+    (** [anchor t2 t1] moves triangle [t1] at position (x, depth) in
+        triangle [t2] *)
+
+  val pose: ?x:float -> ?depth:float -> t -> t -> t
+    (** [pose t2 t1] moves triangle [t1] so that its south point is
+        at position (x, depth) in triangle [t2] *)
+  val pose_left: ?x:float -> ?depth:float -> t -> t -> t
+  val pose_right: ?x:float -> ?depth:float -> t -> t -> t
+    (** similarly, using [bottom_left] and [bottom_right] instead of
+        south point *)
+
+end
+
 (** Draw arrows and build new forms of arrows. *)
 module Arrow : sig
   (** Draw simple or complex arrows. *)
@@ -1640,38 +1773,42 @@ module Arrow : sig
         @param pos label position on the path
         @param anchor label anchor *)
 
-  val point_to_point: ?kind: kind -> ?tex: string -> ?pos: float ->
-    ?anchor: Command.position ->
-    ?outd: Path.direction -> ?ind: Path.direction -> Point.t -> Point.t ->
-    Command.t
-    (** Use [point_to_point a b] to draw an arrow from [a] to [b].
-        @param kind the kind of arrow (default is {!triangle_full})
-        @param tex add a LaTeX label
-        @param pos label position on the path
-        @param anchor label anchor
-        @param outd the outgoing direction, at the beginning of the arrow
-        @param ind the ingoing direction, at the end of the arrow *)
+  type ('a,'b) arrow_from_to =
+      ?kind: kind ->
+      ?tex: string ->
+      ?pos: float ->
+      ?anchor: Command.position ->
+      ?style:Path.joint ->
+      ?outd: Path.direction ->
+      ?ind: Path.direction ->
+      ?sep:Num.t ->
+      'a -> 'b -> Command.t
+  (**
+     {ul {- [kind] the kind of arrow (default is {!triangle_full})}
+     {- [tex] add a LaTeX label}
+     {- [pos] label position on the path}
+     {- [anchor] label anchor}
+     {- [outd] the outgoing direction, at the beginning of the arrow}
+     {- [ind] the ingoing direction, at the end of the arrow}
+     {- [sep] add a little separation of the given distance}
+     }
+    *)
 
-  val box_to_box: ?kind: kind -> ?tex: string -> ?pos: float ->
-    ?anchor: Command.position ->
-    ?outd: Path.direction -> ?ind: Path.direction -> Box.t -> Box.t ->
-    Command.t
+
+  val point_to_point: (Point.t,Point.t) arrow_from_to
+    (** Use [point_to_point a b] to draw an arrow from [a] to [b]. *)
+
+  val box_to_box: (Box.t,Box.t) arrow_from_to
     (** Use [box_to_box] to draw an arrow from [a] to [b], stopping at the
         box boundaries. The arguments are the same as those of
         [point_to_point]. *)
 
-  val point_to_box: ?kind: kind -> ?tex: string -> ?pos: float ->
-    ?anchor: Command.position ->
-    ?outd: Path.direction -> ?ind: Path.direction -> Point.t -> Box.t ->
-    Command.t
+  val point_to_box: (Point.t,Box.t) arrow_from_to
     (** Use [point_to_box] to draw an arrow from [a] to [b], stopping at the
         box boundaries. The arguments are the same as those of
         [point_to_point]. *)
 
-  val box_to_point: ?kind: kind -> ?tex: string -> ?pos: float ->
-    ?anchor: Command.position ->
-    ?outd: Path.direction -> ?ind: Path.direction -> Box.t -> Point.t ->
-    Command.t
+  val box_to_point: (Box.t,Point.t) arrow_from_to
     (** Use [box_to_point] to draw an arrow from [a] to [b], stopping at the
         box boundaries. The arguments are the same as those of
         [point_to_point]. *)
@@ -1807,17 +1944,18 @@ module Helpers : sig
   val draw_simple_arrow :
     ?color:Color.t -> ?pen:Pen.t -> ?dashed:Dash.t ->
     ?style:Path.joint -> ?outd:Path.direction -> ?ind:Path.direction ->
+    ?sep:Num.t ->
     Point.t -> Point.t -> Command.t
   val draw_label_arrow :
     ?color:Color.t -> ?pen:Pen.t -> ?dashed:Dash.t ->
     ?style:Path.joint -> ?outd:Path.direction -> ?ind:Path.direction ->
-    ?pos:Command.position -> Picture.t ->
-    Point.t -> Point.t -> Command.t
+    ?pos:Command.position -> ?sep:Num.t ->
+    Picture.t -> Point.t -> Point.t -> Command.t
   val draw_labelbox_arrow :
     ?color:Color.t -> ?pen:Pen.t -> ?dashed:Dash.t ->
     ?style:Path.joint -> ?outd:Path.direction -> ?ind:Path.direction ->
-    ?pos:Command.position -> Box.t ->
-    Point.t -> Point.t -> Command.t
+    ?pos:Command.position -> ?sep:Num.t ->
+    Box.t -> Point.t -> Point.t -> Command.t
   val box_arrow :
     ?within:Box.t -> ?color:Color.t -> ?pen:Pen.t -> ?dashed:Dash.t ->
     ?style:Path.joint -> ?outd:Path.direction -> ?ind:Path.direction ->
@@ -1836,20 +1974,21 @@ module Helpers : sig
   val box_label_arrow :
     ?within:Box.t -> ?color:Color.t -> ?pen:Pen.t -> ?dashed:Dash.t ->
     ?style:Path.joint -> ?outd:Path.direction -> ?ind:Path.direction ->
-    ?sep:Num.t ->
-    ?pos:Command.position -> Picture.t ->
-    Box.t -> Box.t -> Command.t
+    ?sep:Num.t -> ?pos:Command.position ->
+    Picture.t -> Box.t -> Box.t -> Command.t
   val box_label_line :
     ?within:Box.t -> ?color:Color.t -> ?pen:Pen.t -> ?dashed:Dash.t ->
     ?style:Path.joint -> ?outd:Path.direction -> ?ind:Path.direction ->
-    ?sep:Num.t ->
-    ?pos:Command.position -> Picture.t ->
-    Box.t -> Box.t -> Command.t
+    ?sep:Num.t -> ?pos:Command.position ->
+    Picture.t -> Box.t -> Box.t -> Command.t
   val box_labelbox_arrow :
     ?within:Box.t -> ?color:Color.t -> ?pen:Pen.t -> ?dashed:Dash.t ->
     ?style:Path.joint -> ?outd:Path.direction -> ?ind:Path.direction ->
-    ?sep:Num.t ->
-    ?pos:Command.position -> Box.t ->
+    ?sep:Num.t -> ?pos:Command.position ->
+    Box.t -> Box.t -> Box.t -> Command.t
+  val box_pointer_arrow :
+    ?within:Box.t -> ?color:Color.t -> ?pen:Pen.t -> ?dashed:Dash.t ->
+    ?style:Path.joint -> ?outd:Path.direction -> ?ind:Path.direction ->
     Box.t -> Box.t -> Command.t
 (***
   val hboxjoin :
@@ -2023,6 +2162,11 @@ module Tree_adv : sig
   val fold : ('a -> 'b -> 'a) -> 'a -> 'b t -> 'a
     (** Traverse the tree in a bottom-up, left-to-right order *)
 
+  val fold_child : ('a -> 'b -> 'b -> 'a) -> 'a -> 'b t -> 'a
+    (** Traverse the pairs of parent-child in the tree in a bottom-up,
+        left-to-right order *)
+
+
   val filter : ('a -> bool) -> 'a t -> 'a t
     (** filter f t
         If for a node n of t f n is false then it doesn't appear in the result
@@ -2383,8 +2527,6 @@ end
 (* Misc does not appear in the documentation *)
 (**/**)
 module Misc : sig
-  val write_to_file : string -> (out_channel -> 'a) -> unit
-  val write_to_formatted_file : string -> (Format.formatter -> 'a) -> unit
   val print_option :
     string ->
     (Format.formatter -> 'a -> unit) -> Format.formatter -> 'a option -> unit
@@ -2398,7 +2540,7 @@ module Misc : sig
       [List.fold_left f acc [i; i +1; .. j] ],
      where i <= j *)
   val call_cmd :
-    ?inv:bool -> ?outv:bool -> ?verbose:bool -> string -> int * string
+    ?inv:bool -> ?outv:bool -> ?verbose:bool -> string -> int
 
 end
 (**/**)
@@ -2406,24 +2548,23 @@ end
 (** Functions to generate Metapost files *)
 module Metapost : sig
 
+(**/**) (** This code is deprecated. *)
   val set_filename_prefix : string -> unit
     (** Add to the filename given to the emit function this prefix.
         This function is here just for convenience *)
 
+  (* I believe these functions were only used internally. Commented. *)
+  (*
   val generate_mp :
     string ->
     ?prelude:string ->
     ?eps:bool ->
     (int * Command.t) list -> unit
+    *)
 
   val generate :
-    string ->
-    ?prelude:string ->
-    ?pdf:bool ->
-    ?eps:bool ->
-    ?verbose:bool ->
-    ?clean:bool ->
-    (int * Command.t) list -> unit
+    ?prelude:string -> ?verbose:bool -> ?clean:bool ->
+    string -> (string * Command.t) list -> unit
 
   val emit : string -> Command.t -> unit
   val dump :
@@ -2438,10 +2579,47 @@ module Metapost : sig
 	names specified to [emit]. The file suffix is [.mps] if [pdf] is
 	set, and [.1] otherwise. *)
 
+  type job = string * Command.t
+  type jobs = job list
+  (** Generate files of corresponding type, using the argument of type [jobs],
+     and return information about the created files *)
+(* (* FIXME export this function, too *)
+  val mp :
+    string -> ?prelude:string -> jobs -> File.t * string Misc.IntMap.t
+*)
+  val mps :
+    ?prelude:string -> ?verbose:bool -> string -> jobs -> File.t list
+  val pdf :
+    ?prelude:string -> ?verbose:bool -> string -> jobs -> File.t list
+  val png :
+    ?prelude:string -> ?verbose:bool -> string -> jobs -> File.t list
+
+  (** Same as above, but use a temporary directory *)
+(* (* FIXME export this function, too *)
+  val temp_mp :
+    ?prelude:string -> ?verbose:bool -> ?clean:bool -> string -> jobs ->
+      File.t * string Misc.IntMap.t
+*)
+  val temp_mps :
+    ?prelude:string -> ?verbose:bool -> ?clean:bool -> string ->
+      jobs -> File.t list
+  val temp_pdf :
+    ?prelude:string -> ?verbose:bool -> ?clean:bool -> string ->
+      jobs -> File.t list
+  val temp_png :
+    ?prelude:string -> ?verbose:bool -> ?clean:bool -> string ->
+      jobs -> File.t list
+
+  (** Same as above, but use a temporary directory and take jobs from the job
+     queue *)
   val dump_mp :
-    ?prelude:string -> string -> unit
+    ?prelude:string -> ?verbose:bool -> ?clean:bool -> string -> unit
+  val dump_mps :
+    ?prelude:string -> ?verbose:bool -> ?clean:bool -> string -> unit
   val dump_png :
     ?prelude:string -> ?verbose:bool -> ?clean:bool -> string -> unit
+  val dump_pdf :
+    ?prelude:string -> ?verbose:bool -> ?clean:bool -> string -> unit
 
   val read_prelude_from_tex_file : string -> string
     (** read the prelude from a tex file, until the end of file or the text
@@ -2465,16 +2643,7 @@ module Metapost : sig
   val dumpable : unit -> unit
   val depend : string -> unit
 
-end
-
-(** Helper functions to generate test TeX files *)
-module Generate : sig
-  val generate_tex : ?pdf:bool -> string -> string -> string
-    -> (int * 'a) list -> unit
-
-  val generate_tex_cairo : string -> string -> string -> string
-    -> (int * 'a) list -> unit
-
+(**/**)
 end
 
 (** Compute concrete values of numerics, points and paths;
@@ -2485,11 +2654,7 @@ module Concrete : sig
   (** The module of concrete points *)
   module CPoint : sig
 
-    IFDEF CAIRO THEN
-      type t = Cairo.point = { x : float; y : float }
-    ELSE
-      type t = { x : float; y : float }
-    END
+    type t = Ctypes.point
 
     val add : t -> t -> t
     val sub : t -> t -> t
@@ -2576,21 +2741,12 @@ module Concrete : sig
 
   module CTransform :
   sig
-    IFDEF CAIRO THEN
-    type t = Cairo.matrix =
-        { xx : float; yx : float
-        ; xy : float; yy : float; x0 : float; y0 : float; }
-    ELSE
-    type t =
-        { xx : float; yx : float;
-          xy : float; yy : float; x0 : float; y0 : float; }
-    END
+    type t = Ctypes.matrix
   end
 
   (** {2 Compute the concrete value} *)
+
   val float_of_num : Num.t -> float
-  val compute_nums : unit -> (Num.t -> unit) * (unit -> unit)
-    (** The value of a num in bp *)
 
   val cpoint_of_point : Point.t -> CPoint.t
 
@@ -2599,32 +2755,36 @@ module Concrete : sig
   val ctransform_of_transform : Transform.t -> CTransform.t
 
   (** {2 Compute the baselines of a tex} *)
+
   val baselines : string -> float list
 
   (** {2 Simple functions for the opposite operations} *)
 
   val num_of_float : float -> Num.t
-    (** exactly Num.bp *)
+    (** Same as [Num.bp]. *)
 
   val point_of_cpoint : CPoint.t -> Point.t
   val path_of_cpath : CPath.t -> Path.t
 
   val transform_of_ctransform : CTransform.t -> Transform.t
 
-    (** {2 Some options (the mlpost tool takes care of them)} *)
+  (** {2 Some options (the mlpost tool takes care of them)} *)
+
+  (** These functions are deprecated. *)
+
   val set_verbosity : bool -> unit
+    (** (deprecated) *)
 
   val set_prelude : string -> unit
-    (** set_prelude filename uses the prelude of the file filename
-        for compilation of the tex snippets *)
+    (** (deprecated) [set_prelude filename] uses the prelude of the file
+        filename for compilation of the tex snippets *)
 
   val set_prelude2 : string option -> unit
-    (** set_prelude2 prelude uses this prelude
+    (** (deprecated) [set_prelude2 prelude] uses this prelude
         for compilation of the tex snippets *)
 
   val set_t1disasm : string option -> unit
-    (** Deprecated in a next release *)
-
+    (** (deprecated) *)
 end
 
 (** Use the Cairo backend to draw your figures; not always available *)
@@ -2665,4 +2825,22 @@ end
 module Metapost_tool : sig
   val read_prelude_from_tex_file : string -> string
 end
+
+module Version : sig
+  val version : string
+  val date : string
+end
+
+module Mps : sig
+  val dump: unit -> unit
+  val generate : (string * Command.t) list -> unit
+end
 (**/**)
+
+module Defaults : sig
+  val set_prelude : string -> unit
+  val set_filename_prefix : string -> unit
+  val set_required_files : string list -> unit
+  val set_verbosity : bool -> unit
+  val set_t1disasm : string option -> unit
+end
