@@ -15,6 +15,7 @@
 (**************************************************************************)
 
 open Types
+open Point_lib
 
 type t = commandpic
 
@@ -23,31 +24,44 @@ let make l = l
 let empty = mkSeq []
 let bbox pic = mkPABBox pic
 
-let ulcorner pic = mkPTPicCorner pic `Northwest
-let llcorner pic = mkPTPicCorner pic `Southwest
-let urcorner pic = mkPTPicCorner pic `Northeast
-let lrcorner pic = mkPTPicCorner pic `Southeast
+let ulcorner pic =
+  let pic = Compute.commandpic pic in
+  let ll, ur = Picture_lib.bounding_box pic in
+  { x = ll.x ; y = ur.y }
+
+let llcorner pic =
+  let pic = Compute.commandpic pic in
+  let ll, _ = Picture_lib.bounding_box pic in
+  ll
+
+let urcorner pic =
+  let pic = Compute.commandpic pic in
+  let _, ur = Picture_lib.bounding_box pic in
+  ur
+
+let lrcorner pic =
+  let pic = Compute.commandpic pic in
+  let ll, ur = Picture_lib.bounding_box pic in
+  { Point_lib.x = ur.x ; y = ll.y }
 
 let north_west = ulcorner
 let south_west = llcorner
 let north_east = urcorner
 let south_east = lrcorner
 
-let corner_bbox ?(dx=Num.zero) ?(dy=Num.zero) pic = 
+let corner_bbox ?(dx=Num.zero) ?(dy=Num.zero) pic =
   let pdx = Point.pt (dx, Num.zero) in
   let pdy = Point.pt (Num.zero, dy) in
-  Path.pathp ~style:Path.jLine ~cycle:Path.jLine 
+  Path.pathp ~style:Path.jLine ~cycle:Path.jLine
     [Point.add (Point.sub (ulcorner pic) pdx) pdy;
      Point.sub (Point.sub (llcorner pic) pdx) pdy;
      Point.sub (Point.add (lrcorner pic) pdx) pdy;
      Point.add (Point.add (urcorner pic) pdx) pdy]
 
-let transform trl p = 
-  List.fold_left 
-    (fun acc tr -> mkPicture (mkPITransformed acc tr)) p trl 
-
 let ctr pic = Point.segment 0.5 (llcorner pic) (urcorner pic)
 
+let transform tr p = mkPicture (mkPITransformed p tr)
+
 let scale f p = transform [Transform.scaled f] p
 let rotate f p = transform [Transform.rotated f] p
 let shift pt p = transform [Transform.shifted pt] p
@@ -63,13 +77,13 @@ let place_up_right p pic = place urcorner pic p
 let place_bot_left p pic = place llcorner pic p
 let place_bot_right p pic = place lrcorner pic p
 
-let beside p1 p2 = 
+let beside p1 p2 =
   mkSeq [p1; place_up_left (urcorner p1) p2]
 
 let below p1 p2 =
   mkSeq [p1; place_up_left (llcorner p1) p2]
 
-let clip pic pth = 
+let clip pic pth =
   mkPicture (mkPIClip pic pth)
 
 let width p = Point.xpart (Point.sub (urcorner p) (ulcorner p))
@@ -81,7 +95,7 @@ let south p = Point.segment 0.5 (south_east p) (south_west p)
 let west p = Point.segment 0.5 (south_west p) (north_west p)
 let east p = Point.segment 0.5 (north_east p) (south_east p)
 
-let corner pos x = 
+let corner pos x =
   match pos_reduce pos with
   | `Northwest -> north_west x
   | `Northeast -> north_east x
@@ -93,7 +107,7 @@ let corner pos x =
   | `North -> north x
   | `South -> south x
 
-type escaped = 
+type escaped =
     [`Backslash
     |`Underscore]
 
