@@ -15,12 +15,11 @@
 (**************************************************************************)
 
 type point = Types.point
-type num = Types.num
 
 module type Boxlike =
 sig
   type t
-  val width : t -> num
-  val height : t -> num
+  val width : t -> float
+  val height : t -> float
   val set_pos : point -> t -> t
 end
