@@ -38,9 +38,16 @@ let rec bin = function
 let to_box n = Box.tex (sprintf "${2^{%d}}$" n), n
 let t = map to_box (bin 4)
 
-let t = Box.place ~width:(fun (z,_) -> Box.width z)
-              ~height:(fun (z,_) -> Box.height z)
-              ~set_pos:(fun p (z,n) -> Box.center p z, n) t
+module Pl = struct
+  type t = Box.t * int
+  let width (z,_) = Box.width z
+  let height (z,_) = Box.height z
+  let set_pos p (z,n) = Box.center p z, n
+end
+
+module P = Tree_adv.Place(Pl)
+
+let t = P.place t
 
 let adv_fig =
   let (++) = Command.(++) in
@@ -261,7 +268,7 @@ let d7 =
      Command.dotlabel ~pos:`East (Picture.tex "lrcorner") (Picture.lrcorner pic);
     ]
 
-let half pic = Picture.transform [Transform.scaled (bp 0.5)] pic
+let half = Picture.scale (bp 0.5)
 
 let rec right_split n pic =
   if n <= 0 then pic
@@ -271,18 +278,18 @@ let rec right_split n pic =
 
 let d11 =
   let p1 = Picture.transform [Transform.rotated 90.] (Picture.tex "recursion") in
-    right_split 4 p1
+    right_split 3 p1
 
 let rec sierpinski p n =
   if n = 0 then p else
-    let sp = sierpinski p (n-1) in
-    let p = half sp in
+    let p = sierpinski p (n-1) in
+(*     let p = half sp in *)
     let p1 = Picture.beside p p in
       Picture.below p p1
 
 let d12 =
   let p1 = Picture.tex "A" in
-    sierpinski p1 7
+    sierpinski p1 5
 
 
 (** plots *)
@@ -555,7 +562,74 @@ let grid_with_padding_2 =
   in
   seq [Box.draw b; Box.draw (shift (Point.pt (bp 5., bp 5.)) b)]
 
+module Tr = Triangle
+open Color
+open Command
+
+let t40 =
+  Tr.create ~left:0.5 ~depth:(bp 40.) ()
+
+let x =
+  let t = Tr.create ~fill:white ~depth:(bp 13.) () in
+  Tr.tex_root_label "$x$" t
+
+let anchor1 =
+  let t1 = Tr.tex_label ~depth:0.5 "$\\sigma$" t40 in
+  let t2 = Tr.anchor ~depth:1.0 ~x:0.3 t1 x in
+  let t3 = Tr.anchor ~depth:1.0 ~x:0.7 t1 x in
+  Tr.draw t1 ++
+  Tr.draw t2 ++
+  Tr.draw t3
+
+let pose =
+  let t1 = Tr.pose_left ~x:0.2 t40 x in
+  let t2 = Tr.pose_right ~x:0.8 t40 x in
+  let t3 = Tr.pose_right ~x:0.95 ~depth:1.2 t40
+    (Tr.create ~fill:lightred ~depth:13. ()) in
+  Tr.draw t40 ++ Tr.draw t1 ++ Tr.draw t2 ++ Tr.draw t3
+
+let redex1 =
+  let t1 = Tr.create ~left:0.5 ~depth:(bp 40.) () in
+  let t1 = Tr.tex_root_label "$t$" t1 in
+  let t = Tr.tex ~dx:2. ~dy:2. ~fill:lightred "$r$" in
+  let t = Tr.tex_root_label "\\footnotesize$p$" t in
+  let t = Tr.anchor ~depth:0.8 ~x:0.3 t1 t in
+  Tr.draw t1 ++
+  Tr.draw t
+
+let label1 =
+  let t = Tr.create ~left:0.5 ~depth:(bp 40.) () in
+  let t = Tr.tex_label "$\\sigma t$" t in
+  Tr.draw t
+
+let pic1 =
+  let t = Tr.tex ~pen ~fill:lightred "TOT" in
+  Tr.draw t
+
+let x_depth =
+  let t = Tr.create ~left:20. ~right:60. ~depth:70. () in
+  let r = Tr.root t in
+  let depth = 0.7 in
+  let x0 = Tr.x_depth ~depth ~x:0.  t in
+  let x1 = Tr.x_depth ~depth ~x:1.  t in
+  let x  = Tr.x_depth ~depth ~x:0.7 t in
+  let p01 = Path.pathp
+    [Tr.x_depth ~depth ~x:(-0.5) t; Tr.x_depth ~depth ~x:(1.5) t] in
+  let p = Point.add r (Point.mult 1.8 (Point.sub x r)) in
+  let pd = Path.pathp [r; p] in
+  let b = Tr.x_depth ~depth:1. ~x:0.7 t in
+  Tr.draw t ++
+  Command.dotlabel ~pos:`Topleft    (Picture.tex "0") x0 ++
+  Command.dotlabel ~pos:`Topright   (Picture.tex "1") x1 ++
+  Command.dotlabel ~pos:`Bottomleft (Picture.tex "x") x  ++
+  Command.dotlabel ~pos:`Topleft    (Picture.tex "0") r  ++
+  Command.dotlabel ~pos:`Bottomleft (Picture.tex "1") b ++
+  Command.draw ~color:purple p01 ++
+  Command.draw ~color:red    pd ++
+  nop
+
 let figs = [
+(*
   grid_with_padding;
   adv_fig;
   grid_with_padding_2;
@@ -570,16 +644,26 @@ let figs = [
   why_platform;d2;
   block2;
   vblock1;
-  yannick Box.Rect; yannick Box.Patatoid; yannick Box.Patatoid2; d1;  proval;
-  d2sq; d2hsq; d2s; d2c; cheno011; d3; d4;
+  yannick Box.Rect; yannick Box.Patatoid; yannick Box.Patatoid2; d1;
+  d2sq; d2hsq; d2s; d2c;
+  cheno011; d3; d4;
   d7;
-  (* recursion *)
-  d11; d12 ;
+  proval;
+*)
+   d11;
+   d12 ;
+  d14; d13; d5;
+  florence; Box.draw shapes1; Box.draw shapes2;
   farey 17;
   (* other *)
-  florence; Box.draw shapes1; Box.draw shapes2;
-  d14; d13; d5;
-(*   d6 *)
+  d6;
+  Box.draw t40;
+  anchor1;
+  pose;
+  redex1;
+  label1;
+  pic1;
+  x_depth;
 ]
 
 let figs =
@@ -588,26 +672,12 @@ let figs =
 
 (* CM fonts do not scale well *)
 
+(*
 let theprelude = "\\documentclass[a4paper]{article}
 \\usepackage[T1]{fontenc}
 \\usepackage{times}
 "
-
+*)
 
 let () =
-  Metapost.generate_mp ~prelude:theprelude "test/tests.mp" figs;
-  Misc.write_to_formatted_file "test/tests.tex"
-    (fun fmt ->
-      fprintf fmt "\\documentclass[a4paper]{article}@.";
-      fprintf fmt "\\usepackage[T1]{fontenc}@.";
-      fprintf fmt "\\usepackage{times}@.";
-      fprintf fmt "\\usepackage{fullpage}@.";
-      fprintf fmt "\\usepackage[]{graphicx}@.";
-      fprintf fmt "@[<hov 2>\\begin{document}@.";
-      List.iter
-        (fun (i,_) ->
-          fprintf fmt "@\n %i\\quad" i;
-	  fprintf fmt "\\includegraphics[width=\\textwidth,height=\\textheight,keepaspectratio]{tests.%d}" i;
-          fprintf fmt "@\n \\vspace{3cm}@\n"
-        ) figs;
-      fprintf fmt "@]@\n\\end{document}@.")
+  List.iter (fun (i,r) -> Metapost.emit (string_of_int i) r) figs
