@@ -49,34 +49,8 @@ type position_red = [ | hposition_red | vposition_red | corner_red ]
 
 open Hashcons
 
-type num_node =
-  | F of float
-  | NXPart of point
-  | NYPart of point
-  | NAdd of num * num
-  | NSub of num * num
-  | NMult of num * num
-  | NDiv of num * num
-  | NMax of num * num
-  | NMin of num * num
-  | NGMean of num * num
-  | NLength of path
-  | NIfnullthenelse of num * num * num
-
-and num = num_node hash_consed
-
-and point_node =
-  | PTPair of num * num
-  | PTPicCorner of commandpic * corner
-  | PTPointOf of num * path
-  | PTDirectionOf of num * path
-  | PTAdd of point * point
-  | PTSub of point * point
-  | PTMult of num * point
-  | PTRotated of float * point
-  | PTTransformed of point * transform
-
-and point = point_node hash_consed
+type num = float
+type point = Point_lib.t
 
 and on_off_node =
   | On of num
@@ -130,23 +104,8 @@ and path_node =
 
 and path = path_node hash_consed
 
-and matrix =
-    { xx : num; yx : num;
-      xy : num; yy : num; x0 : num; y0 : num; }
-
-and transform_node =
-  | TRRotated of float
-  | TRScaled of num
-  | TRShifted of point
-  | TRSlanted of num
-  | TRXscaled of num
-  | TRYscaled of num
-  | TRZscaled of point
-  | TRReflect of point * point
-  | TRRotateAround of point * float
-  | TRMatrix of matrix
-
-and transform = transform_node hash_consed
+and matrix = Matrix.t
+and transform = Matrix.t list
 
 and dash_node =
   | DEvenly
@@ -201,11 +160,11 @@ and brush_node = {pen : pen option;
 
 and brush = brush_node hash_consed
 
-let hash_float = Hashtbl.hash
+let hash_float : float -> int = Hashtbl.hash
 
-let hash_piccorner = Hashtbl.hash
+let hash_piccorner : corner -> int = Hashtbl.hash
 
-let hash_string = Hashtbl.hash
+let hash_string : string -> int = Hashtbl.hash
 
 let combine n acc = acc * 65599 + n
 
@@ -216,38 +175,27 @@ let combine3 n acc1 acc2 acc3 = combine n (combine acc1 (combine acc2 acc3))
 let combine4 n acc1 acc2 acc3 acc4 = combine n (combine3 acc1 acc2 acc3 acc4)
 
 
-let num = function
-  | F f -> combine 1 (hash_float f)
-  | NXPart p -> combine 2 p.hkey
-  | NYPart p -> combine 3 p.hkey
-  | NAdd(n,m) -> combine2 4 n.hkey m.hkey
-  | NSub(n,m) -> combine2 5 n.hkey m.hkey
-  | NMult(n,m) -> combine2 6 n.hkey m.hkey
-  | NDiv(n,m) -> combine2 7 n.hkey m.hkey
-  | NMax(n,m) -> combine2 8 n.hkey m.hkey
-  | NMin(n,m) -> combine2 9 n.hkey m.hkey
-  | NGMean(n,m) -> combine2 10 n.hkey m.hkey
-  | NLength p -> combine 11 p.hkey
-  | NIfnullthenelse (n,n1,n2) -> combine3 12 n.hkey n1.hkey n2.hkey
+let num = hash_float
 
+let point { Point_lib.x = x ; y = y } = combine2 12 (num x) (num y)
 
-let point = function
-  | PTPair(n,m) -> combine2 12 n.hkey m.hkey
-  | PTPicCorner(p,pc) -> combine2 13 p.hkey (hash_piccorner pc)
-  | PTPointOf(f,p) -> combine2 14 (hash_float f) p.hkey
-  | PTDirectionOf(f,p) -> combine2 15 (hash_float f) p.hkey
-  | PTAdd(p,q) -> combine2 16 p.hkey q.hkey
-  | PTSub(p,q) -> combine2 17 p.hkey q.hkey
-  | PTMult(n,q) -> combine2 18 n.hkey q.hkey
-  | PTRotated(f,p) -> combine2 19 (hash_float f) p.hkey
-  | PTTransformed(p,tr) -> combine2 20 p.hkey tr.hkey
+let transform' { Matrix.xx = xx; yx = yx; xy = xy; yy = yy; x0 = x0; y0 = y0} =
+  combine4 52 (hash_float xx) (hash_float yx) (hash_float xy)
+    (combine3 53 (hash_float yy) (hash_float x0) (hash_float y0))
+
+let rec hash_list h l =
+  match l with
+  | [] -> 54
+  | x::xs -> combine2 55 (h x) (hash_list h xs)
+
+let transform = hash_list transform'
 
 let on_off = function
-  | On n -> combine 65 n.hkey
-  | Off n -> combine 66 n.hkey
+  | On n -> combine 65 (num n)
+  | Off n -> combine 66 (num n)
 
 let direction = function
-  | Vec p -> combine 61 p.hkey
+  | Vec p -> combine 61 (point p)
   | Curl f -> combine 62 (hash_float f)
   | NoDir -> 63
 
@@ -256,10 +204,10 @@ let joint = function
   | JCurve -> 68
   | JCurveNoInflex -> 69
   | JTension(f1,f2) -> combine2 70 (hash_float f1) (hash_float f2)
-  | JControls(p1,p2) -> combine2 71 p1.hkey p2.hkey
+  | JControls(p1,p2) -> combine2 71 (point p1) (point p2)
 
 let knot k =
-  combine3 64 k.knot_in.hkey k.knot_p.hkey k.knot_out.hkey
+  combine3 64 k.knot_in.hkey (point k.knot_p) k.knot_out.hkey
 
 let metapath = function
   | MPAConcat(k,j,p) ->
@@ -276,7 +224,7 @@ let path = function
   | PAHalfCircle -> 25
   | PAQuarterCircle -> 26
   | PAUnitSquare -> 27
-  | PATransformed(p,tr) -> combine2 28 p.hkey tr.hkey
+  | PATransformed(p,tr) -> combine2 28 p.hkey (transform tr)
   | PACutAfter(p,q) -> combine2 32 p.hkey q.hkey
   | PACutBefore(p,q) -> combine2 33 p.hkey q.hkey
   | PABuildCycle l ->
@@ -285,23 +233,9 @@ let path = function
       combine3 36 (hash_float f1) (hash_float f2) p.hkey
   | PABBox p -> combine 37 p.hkey
 
-let transform = function
-  | TRRotated f -> combine 52 (hash_float f)
-  | TRScaled n -> combine 53 n.hkey
-  | TRShifted p -> combine 57 p.hkey
-  | TRSlanted n -> combine 54 n.hkey
-  | TRXscaled n -> combine 55 n.hkey
-  | TRYscaled n -> combine 56 n.hkey
-  | TRZscaled p -> combine 58 p.hkey
-  | TRReflect(p,q) -> combine2 59 p.hkey q.hkey
-  | TRRotateAround(p,q) -> combine2 60 p.hkey (hash_float q)
-  | TRMatrix m ->
-      List.fold_left (fun acc n -> combine2 63 acc n.hkey) 61
-        [m.x0;m.y0;m.xx;m.xy;m.yx;m.yy]
-
 let picture = function
   | PITex s -> combine 38 (hash_string s)
-  | PITransformed(p,tr) -> combine2 40 p.hkey tr.hkey
+  | PITransformed(p,tr) -> combine2 40 p.hkey (transform tr)
   | PIClip(p,q) -> combine2 42 p.hkey q.hkey
 
 let commandpic = function
@@ -312,8 +246,8 @@ let commandpic = function
 let dash = function
   | DEvenly -> 72
   | DWithdots -> 73
-  | DScaled(f,d) -> combine2 74 (hash_float f) d.hkey
-  | DShifted(p,d) -> combine2 75 p.hkey d.hkey
+  | DScaled(f,d) -> combine2 74 (num f) d.hkey
+  | DShifted(p,d) -> combine2 75 (point p) d.hkey
   | DPattern l ->
       List.fold_left (fun acc o -> combine2 76 acc o.hkey) 77 l
 
@@ -321,7 +255,7 @@ let pen = function
   | PenCircle -> 78
   | PenSquare -> 79
   | PenFromPath p -> combine 80 p.hkey
-  | PenTransformed(p,tr) -> combine2 81 p.hkey tr.hkey
+  | PenTransformed(p,tr) -> combine2 81 p.hkey (transform tr)
 
 let hash_opt f = function
   | None -> 83
@@ -339,9 +273,9 @@ let command = function
   | CDraw(pa,b) ->
       combine2 43 pa.hkey b.hkey
   | CFill(p,c) -> combine2 46 p.hkey (hash_color c)
-  | CLabel(pic,pos,poi) -> combine3 47 pic.hkey (hash_position pos) poi.hkey
+  | CLabel(pic,pos,poi) -> combine3 47 pic.hkey (hash_position pos) (point poi)
   | CDotLabel(pic,pos,poi) ->
-      combine3 48 pic.hkey (hash_position pos) poi.hkey
+      combine3 48 pic.hkey (hash_position pos) (point poi)
   | CExternalImage (filename,spec) -> combine2 52 (hash_string filename)
       (hash_spec_image spec)
 
@@ -355,6 +289,10 @@ let brush b =
 let eq_float (f1:float) (f2:float) =
   Pervasives.compare f1 f2 == 0
 
+let eq_num = eq_float
+let eq_point (a : point) (b : point) = Pervasives.compare a b == 0
+let eq_matrix (a : matrix) (b : matrix) = Pervasives.compare a b == 0
+
 (* we enforce to use physical equality only on hash-consed data
    of the same type *)
 let eq_hashcons (x:'a hash_consed) (y:'a hash_consed) =
@@ -366,6 +304,14 @@ let rec eq_hashcons_list (x:'a hash_consed list) (y:'a hash_consed list) =
     | h1::t1,h2::t2 -> h1 == h2 && eq_hashcons_list t1 t2
     | _ -> false
 
+let rec eq_list eq x y =
+  match x, y with
+  | [], [] -> true
+  | x::xs, y::ys -> eq x y && eq_list eq xs ys
+  | _, _ -> false
+
+let eq_transform = eq_list eq_matrix
+
 let eq_opt f o1 o2 =
   match o1,o2 with
     | None,None -> true
@@ -381,7 +327,7 @@ let eq_pen_node p1 p2 =
     | PenFromPath p1, PenFromPath p2 ->
 	eq_hashcons p1 p2
     | PenTransformed(p1,tr1), PenTransformed(p2,tr2) ->
-	eq_hashcons p1 p2 && eq_hashcons tr1 tr2
+	eq_hashcons p1 p2 && eq_transform tr1 tr2
     | _ -> false
 
 let eq_dash_node d1 d2 =
@@ -389,9 +335,9 @@ let eq_dash_node d1 d2 =
   | DEvenly, DEvenly
   | DWithdots, DWithdots -> true
   | DScaled(f1,d1), DScaled(f2,d2) ->
-      eq_hashcons f1 f2 && eq_hashcons d1 d2
+      eq_num f1 f2 && eq_hashcons d1 d2
   | DShifted(p1,d1), DShifted(p2,d2) ->
-      eq_hashcons p1 p2 && eq_hashcons d1 d2
+      eq_point p1 p2 && eq_hashcons d1 d2
   | DPattern l1, DPattern l2 ->
       eq_hashcons_list l1 l2
   | _ -> false
@@ -404,50 +350,12 @@ let eq_brush_node b1 b2 =
 let eq_on_off o1 o2 =
   match o1,o2 with
     | Off n1, Off n2
-    | On n1, On n2 -> eq_hashcons n1 n2
+    | On n1, On n2 -> eq_num n1 n2
     | _ -> false
 
 let eq_position (p1:position) (p2:position) =
   p1 == p2 (* correct because this type contains only constants *)
 
-let eq_num_node n1 n2 =
-  match n1,n2 with
-    | F f1, F f2 -> eq_float f1 f2
-    | NXPart p1, NXPart p2
-    | NYPart p1, NYPart p2 -> eq_hashcons p1 p2
-    | NAdd(n11,n12),NAdd(n21,n22)
-    | NSub(n11,n12),NSub(n21,n22)
-    | NMult(n11,n12),NMult(n21,n22)
-    | NDiv(n11,n12),NDiv(n21,n22)
-    | NMax(n11,n12),NMax(n21,n22)
-    | NMin(n11,n12),NMin(n21,n22)
-    | NGMean(n11,n12),NGMean(n21,n22)
-	-> eq_hashcons n11 n21 && eq_hashcons n12 n22
-    | NLength p1, NLength p2 -> eq_hashcons p1 p2
-    | _ -> false
-
-let eq_point_node p1 p2 =
-  match p1,p2 with
-    | PTPair(n11,n12),PTPair(n21,n22) ->
-	eq_hashcons n11 n21 && eq_hashcons n12 n22
-    | PTPicCorner(pic1,corn1), PTPicCorner(pic2,corn2) ->
-	eq_hashcons pic1 pic2 &&
-        eq_position (corn1 :> position) (corn2 :> position)
-    | PTPointOf(n1,p1), PTPointOf(n2,p2)
-    | PTDirectionOf(n1,p1), PTDirectionOf(n2,p2)
-	-> eq_hashcons n1 n2 && eq_hashcons p1 p2
-    | PTAdd(p11,p12),PTAdd(p21,p22)
-    | PTSub(p11,p12),PTSub(p21,p22)
-	-> eq_hashcons p11 p21 && eq_hashcons p12 p22
-    | PTMult(n1,p1),PTMult(n2,p2) ->
-	eq_hashcons n1 n2 && eq_hashcons p1 p2
-    | PTRotated(f1,p1),PTRotated(f2,p2) ->
-	eq_float f1 f2 && eq_hashcons p1 p2
-    | PTTransformed(p1,tr1), PTTransformed(p2,tr2) ->
-	eq_hashcons p1 p2 && eq_hashcons tr1 tr2
-    | _ -> false
-
-
 let eq_metapath_node p1 p2 =
   match p1,p2 with
     | MPAConcat(k1,j1,p1),MPAConcat(k2,j2,p2) ->
@@ -469,14 +377,14 @@ let eq_path_node p1 p2 =
     | PAUnitSquare, PAUnitSquare
 	-> true
     | PATransformed(p1,tr1),PATransformed(p2,tr2) ->
-	eq_hashcons p1 p2 && eq_hashcons tr1 tr2
+	eq_hashcons p1 p2 && eq_transform tr1 tr2
     | PACutAfter(p11,p12),PACutAfter(p21,p22)
     | PACutBefore(p11,p12),PACutBefore(p21,p22)
 	-> eq_hashcons p11 p21 && eq_hashcons p12 p22
     | PABuildCycle(l1),PABuildCycle(l2) ->
 	eq_hashcons_list l1 l2
     | PASub(f11,f12,p1), PASub(f21,f22,p2) ->
-	eq_hashcons f11 f21 && eq_hashcons f12 f22 && eq_hashcons p1 p2
+	eq_num f11 f21 && eq_num f12 f22 && eq_hashcons p1 p2
     | PABBox(p1), PABBox(p2) ->
 	eq_hashcons p1 p2
     | _ -> false
@@ -488,36 +396,14 @@ let eq_picture_node p1 p2 =
         (* it actually happens that the same text appears twice *)
         s1<>"" && s1=s2
     | PITransformed(p1,tr1), PITransformed(p2,tr2) ->
-	eq_hashcons p1 p2 && eq_hashcons tr1 tr2
+	eq_hashcons p1 p2 && eq_transform tr1 tr2
     | PIClip(pi1,pa1), PIClip(pi2,pa2) ->
 	eq_hashcons pi1 pi2 && eq_hashcons pa1 pa2
     | _ -> false
 
-let eq_transform_node t1 t2 =
-  match t1,t2 with
-  | TRRotated f1, TRRotated f2 -> eq_float f1 f2
-  | TRScaled n1, TRScaled n2
-  | TRSlanted n1, TRSlanted n2
-  | TRXscaled n1, TRXscaled n2
-  | TRYscaled n1, TRYscaled n2 -> eq_hashcons n1 n2
-  | TRShifted p1, TRShifted p2
-  | TRZscaled p1, TRZscaled p2 -> eq_hashcons p1 p2
-  | TRReflect(p11,p12), TRReflect(p21,p22) ->
-      eq_hashcons p11 p21 && eq_hashcons p12 p22
-  | TRRotateAround(p1,f1), TRRotateAround(p2,f2) ->
-      eq_hashcons p1 p2 && eq_float f1 f2
-  | TRMatrix m1, TRMatrix m2 ->
-      eq_hashcons m1.x0 m2.x0
-      &&  eq_hashcons m1.y0 m2.y0
-      &&  eq_hashcons m1.xx m2.xx
-      &&  eq_hashcons m1.xy m2.xy
-      &&  eq_hashcons m1.yx m2.yx
-      &&  eq_hashcons m1.yy m2.yy
-  | _ -> false
-
 let eq_knot_node k1 k2 =
   eq_hashcons k1.knot_in k2.knot_in &&
-  eq_hashcons k1.knot_p k2.knot_p &&
+  eq_point k1.knot_p k2.knot_p &&
   eq_hashcons k1.knot_out k2.knot_out
 
 let eq_joint_node j1 j2 =
@@ -528,12 +414,12 @@ let eq_joint_node j1 j2 =
   | JTension(f11,f12), JTension(f21,f22) ->
       eq_float f11 f21 && eq_float f12 f22
   | JControls(p11,p12), JControls(p21,p22) ->
-      eq_hashcons p11 p21 && eq_hashcons p12 p22
+      eq_point p11 p21 && eq_point p12 p22
   | _ -> false
 
 let eq_direction_node d1 d2 =
   match d1,d2 with
-    | Vec p1, Vec p2 -> eq_hashcons p1 p2
+    | Vec p1, Vec p2 -> eq_point p1 p2
     | Curl f1, Curl f2 -> eq_float f1 f2
     | NoDir, NoDir -> true
     | _ -> false
@@ -546,7 +432,7 @@ let eq_command_node c1 c2 =
       eq_hashcons p1 p2 && eq_opt eq_color c1 c2
   | CLabel(pic1,pos1,poi1), CLabel(pic2,pos2,poi2)
   | CDotLabel(pic1,pos1,poi1), CDotLabel(pic2,pos2,poi2) ->
-      eq_hashcons pic1 pic2 && eq_position pos1 pos2 && eq_hashcons poi1 poi2
+      eq_hashcons pic1 pic2 && eq_position pos1 pos2 && eq_point poi1 poi2
   | _ -> false
 
 let eq_commandpic_node p1 p2 =
@@ -562,70 +448,6 @@ let eq_commandpic_node p1 p2 =
 
 let unsigned f x = (f x) land 0x3FFFFFFF
 
-module HashNum =
-  Hashcons.Make(struct type t = num_node
-		       let equal = eq_num_node
-		       let hash = unsigned num end)
-
-let hashnum_table = HashNum.create 257;;
-let hashnum = HashNum.hashcons hashnum_table
-
-let mkF f = hashnum (F f)
-let mkNAdd n1 n2 = hashnum (NAdd(n1,n2))
-let mkNSub n1 n2 = hashnum (NSub(n1,n2))
-let mkNMult n1 n2 = hashnum (NMult(n1,n2))
-let mkNDiv n1 n2 = hashnum (NDiv(n1,n2))
-let mkNMax n1 n2 = hashnum (NMax(n1,n2))
-let mkNMin n1 n2 = hashnum (NMin(n1,n2))
-let mkNGMean n1 n2 = hashnum (NGMean(n1,n2))
-let mkNXPart p = hashnum (NXPart p)
-let mkNYPart p = hashnum (NYPart p)
-let mkNLength p = hashnum (NLength p)
-let mkNIfnullthenelse p n1 n2 = hashnum (NIfnullthenelse (p,n1,n2))
-
-(* point *)
-
-
-module HashPoint =
-  Hashcons.Make(struct type t = point_node
-		       let equal = eq_point_node
-		       let hash = unsigned point end)
-
-let hashpoint_table = HashPoint.create 257;;
-let hashpoint = HashPoint.hashcons hashpoint_table
-
-let mkPTPair f1 f2 = hashpoint (PTPair(f1,f2))
-let mkPTAdd p1 p2 = hashpoint (PTAdd(p1,p2))
-let mkPTSub p1 p2 = hashpoint (PTSub(p1,p2))
-let mkPTMult x y = hashpoint (PTMult(x,y))
-let mkPTRotated x y = hashpoint (PTRotated(x,y))
-let mkPTTransformed x y = hashpoint (PTTransformed(x,y))
-let mkPTPointOf f p = hashpoint (PTPointOf(f,p))
-let mkPTDirectionOf f p = hashpoint (PTDirectionOf(f,p))
-let mkPTPicCorner x y = hashpoint (PTPicCorner(x,y))
-
-(* transform *)
-
-
-module HashTransform =
-  Hashcons.Make(struct type t = transform_node
-		       let equal = eq_transform_node
-		       let hash = unsigned transform end)
-
-let hashtransform_table = HashTransform.create 257;;
-let hashtransform = HashTransform.hashcons hashtransform_table
-
-let mkTRScaled n = hashtransform (TRScaled n)
-let mkTRXscaled n = hashtransform (TRXscaled n)
-let mkTRYscaled n = hashtransform (TRYscaled n)
-let mkTRZscaled pt = hashtransform (TRZscaled pt)
-let mkTRRotated f = hashtransform (TRRotated f)
-let mkTRShifted pt = hashtransform (TRShifted pt)
-let mkTRSlanted n = hashtransform (TRSlanted n)
-let mkTRReflect pt1 pt2 = hashtransform (TRReflect(pt1,pt2))
-let mkTRRotateAround pt f = hashtransform (TRRotateAround(pt,f))
-let mkTRMatrix m = hashtransform (TRMatrix m)
-
 (* knot *)
 
 module HashKnot =
@@ -852,9 +674,3 @@ let opposite_position (x: position): position_red =
     | `Northeast -> `Southwest
     | `Southwest -> `Northeast
     | `Southeast -> `Northwest
-
-
-let q_verbosity = Queue.create ()
-let add_set_verbosity x = Queue.add x q_verbosity
-
-let set_verbosity b = Queue.iter (fun x -> x b) q_verbosity
